#include "MyDetectorConstruction.hh"
#include "MyScoreParameterisation.hh"
#include "G4Box.hh"
#include "G4Colour.hh"
#include "G4LogicalVolume.hh"
#include "G4Material.hh"
#include "G4PVParameterised.hh"
#include "G4PVPlacement.hh"
#include "G4RunManager.hh"
#include "G4Tubs.hh"
#include "G4Sphere.hh"
#include "G4VisAttributes.hh"
#include "G4NistManager.hh"
#include "G4SystemOfUnits.hh"
#include "G4PhysicalConstants.hh"

MyDetectorConstruction::MyDetectorConstruction()
  :fpWorldLogical(0)
  ,fpWorldPhysical(0)
{}

MyDetectorConstruction::~MyDetectorConstruction() {}

G4VPhysicalVolume* MyDetectorConstruction::Construct()
{
  // Material Definition
  DefineMaterials();  

  // Geometry Definition
  SetupGeometry();   

  // Return world volume
  return fpWorldPhysical;  
}

void MyDetectorConstruction::DefineMaterials()
{
  G4String symbol;             
  G4double a, z, density;     
  G4int ncomponents;
  G4double fractionmass;
  
  // Define simple materials
  // Define beryllium, silicon and iron 

  new G4Material("Beryllium", z=4.,  a=9.012182*g/mole, density=1.8480*g/cm3);
  new G4Material("Silicon",   z=14., a=28.0855*g/mole,  density=2.330*g/cm3);
  new G4Material("Iron",      z=26., a=55.845*g/mole,   density=7.87*g/cm3);

  new G4Material("Titanium",  z=22., a=47.90*g/mole,    density=4.540*g/cm3);
    
  // Define elements
  G4Element* N = new G4Element("Nitrogen", symbol="N", z=7., a=14.01*g/mole);
  G4Element* O = new G4Element("Oxygen",   symbol="O", z=8., a=16.00*g/mole);
  
  // Define air
  G4Material* air = new G4Material("Air", density= 1.290*mg/cm3, ncomponents=2);
  air->AddElement(N, fractionmass=0.7);
  air->AddElement(O, fractionmass=0.3);
  
  // Define vacuum
  G4Material* vacuum = new G4Material("Vacuum", density= 1.e-5*g/cm3, 
				      ncomponents=1, kStateGas, STP_Temperature, 
				      2.e-2*bar);
  
  vacuum->AddMaterial(air, fractionmass=1.);
  
  // Dump material information
  G4cout << *(G4Material::GetMaterialTable()) << G4endl;
}

void MyDetectorConstruction::SetupGeometry()
{
  // NIST definition of air
  //  G4Material* air = G4Material::GetMaterial("Air");

  G4NistManager* man = G4NistManager::Instance();
  G4Material* air = man->FindOrBuildMaterial("G4_AIR");  
  G4cout << air << G4endl;

  // World volume
  G4Box* worldSolid = new G4Box("World_Solid",           // Name
				2.0*m, 2.0*m, 2.0*m);    // Half lengths
  
  fpWorldLogical = new G4LogicalVolume(worldSolid,	 // Solid
				       air,	         // Material
				       "World_Logical"); // Name
  
  fpWorldPhysical = new G4PVPlacement(0,	         // Rotation matrix pointer
				      G4ThreeVector(),   // Translation vector
				      fpWorldLogical,	 // Logical volume
				      "World_Physical",	 // Name
				      0,		 // Mother volume
				      false,		 // Unused boolean parameter
				      0);		 // Copy number
  ////////////////////////////////////////////////////////////////////////
  // Beam Window (BW)
  G4Material* titanium = G4Material::GetMaterial("Titanium");
  
  G4VSolid* beamWindowSolid = new G4Tubs("BeamWindow_Solid", 0.*cm, 2.0*cm,
					 0.0065*cm, 0.*deg, 360.*deg);
  
  G4LogicalVolume* beamWindowLogical = 
  new G4LogicalVolume(beamWindowSolid, titanium, "BeamWindow_Logical");
  
  new G4PVPlacement(0, G4ThreeVector(0.,0.,-(3.0*cm+ 0.0065*cm)),
		    beamWindowLogical, "BeamWindow_Physical", 
		    fpWorldLogical, false, 0);
  
  ////////////////////////////////////////////////////////////////////////
  // Beam Pipe 
  G4Material* vacuum = G4Material::GetMaterial("Vacuum");
  
  G4VSolid* beamPipeSolid = new G4Tubs("BeamPipe_Solid", 0.*cm, 5.0*cm, 5.0*cm, 
				       0.*deg, 360.*deg);
  
  G4LogicalVolume* beamPipeLogical = 
  new G4LogicalVolume(beamPipeSolid, vacuum, "BeamPipe_Logical");
  
  new G4PVPlacement(0, G4ThreeVector(0.,0., -(3.013*cm+5.0*cm)),
		    beamPipeLogical, "BeamPipe_Physical",
		    fpWorldLogical, false, 0);
  
 /////////////////////////////////////////////////////////////////////////
 // Beryllium target

  G4Material* beryllium = G4Material::GetMaterial("Beryllium");
  G4VSolid*    targetSolid =
          new G4Tubs("Target_Solid",  // its name
               0.0*cm,                // its inner radius
             3.63*cm,                 // its outer radius
               6.31/2.0*cm,           // its half length
            0.0*deg,                  // starting phi angle
           360.0*deg);                // angle of the segment

  G4LogicalVolume* targetLogical =
       new G4LogicalVolume(targetSolid,         // its solid
                           beryllium,           // its material
                            "Target_Logical");  // its name
                                                     
 
  new G4PVPlacement(0,                                    // no rotation
                    G4ThreeVector(0.0,0.0,6.31/2.0*cm),   // transition
                    targetLogical,                        // current logical volume
                    "Target_Physical",                    // its anme
                    fpWorldLogical,                       // mother logical volume
                    false,                                // true not implemented
                    0);                                   // copy number 

/////////////////////////////////////////////////////////////////////////////
//  Iron window
  G4Material* iron = G4Material::GetMaterial("Iron");

  G4VSolid*    chamberSolid =
          new G4Tubs("Chamber_Solid",  // its name
                   0.0*cm,             // its inner radius   
                   2.0*cm,             // its outer radius
                    0.0051/2.0*cm,     // its half length
                    0.0*deg,           // starting phi angle
                    360.0*deg);        // angle of the segment

  G4LogicalVolume* chamberLogical =
       new G4LogicalVolume(chamberSolid,   // its solid
                   iron,                   // its material
                       "Chamber_Logical"); // its name
                                                     
  new G4PVPlacement(0,                          // no rotation
              G4ThreeVector(0.0,0.0,-1.0*cm),   // transition
                         chamberLogical,        // current logical volume
                     "Chamber_Physical",        // its name
                         fpWorldLogical,        // mother logical volume
                                  false,        // true not implemented
                                   0);          // copy number
/////////////////////////////////////////////////////////////////////////////
// silicon monitor
 
  G4Material* silicon = G4Material::GetMaterial("Silicon");

  G4VSolid*    monitorSolid =
      new G4Tubs("Monitor_Solid",   // its name
                0.0*cm,             // its inner radius
                2.0*cm,             // its outer radius
                0.01/2.0*cm,        // its half length
                0.0*deg,            // starting phi angle
              360.0*deg);           // angle of the segment

  G4LogicalVolume* monitorLogical =
         new G4LogicalVolume(monitorSolid,       // its solid
                        silicon,                 // its material
                           "Monitor_Logical");   // its name
                                                     
  new G4PVPlacement(0,                           // no rotation
              G4ThreeVector(0.0,0.0,-2.0*cm),    // transition
                           monitorLogical,       // current logical volume
                        "Monitor_Physical",      // its anme
                          fpWorldLogical,        // mother logical volume
                                   false,        // true not implemented
                                     0);         // copy number
 ////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
  // sphere
  G4double innerRadius = 40.0*cm;
  G4double outerRadius = 40.01*cm;

  G4VSolid* sphereSolid =
    new G4Sphere("Sphere_Solid",                  //its name
                 innerRadius,                     //its inner radius
                 outerRadius,                     //its outer radius
                 0.0*deg,                         //its starting phi angle
                 360.0*deg,                       //its total phi angle
                 0.0*deg,                         //its starting theta angle
                 90.0*deg);                       //its total theta angle

  G4LogicalVolume* sphereLogical =
    new G4LogicalVolume(sphereSolid,              //its solid
                        air,                      //its material
                        "Sphere_logical");        //its name

  new G4PVPlacement(0,                         //no rotation
                    G4ThreeVector(),           //no transition
                    sphereLogical,             //its logical volume
                    "Sphere_physical",         //its name
                    fpWorldLogical,            //its mather volume
                    false,                     //true not implemented
                    0);                        // copy number

///////////////////////////////////////////////////////////////////////////
 // score 
 // G4int nRings = 15;
 // new MyScoreParameterisation(innerRadius,outerRadius,nRings);                                
////////////////////////////////////////////////////////////////////////
  
  G4int nRings=15;
  G4double deltaTheta=90.0*deg/nRings;
 
  G4VSolid * scoreSolid =
     new G4Sphere("Score_Solid",          //its name
                 innerRadius,             //its inner radius
                 outerRadius,             //its outer radius
                 0.0*deg,                 //its starting phi angle
                 360.0*deg,               //its total phi angle
                 0.0*deg,                 //its starting theta angle
                 deltaTheta);             //its total theta angle
        
  G4LogicalVolume* scoreLogical =
    new G4LogicalVolume(scoreSolid,      //its solid
                        air,             //its material
                        "Score_Logical");//its name
 
  MyScoreParameterisation* param =
    new MyScoreParameterisation(innerRadius,outerRadius,nRings);

  new G4PVParameterised("Score_Physical",   //its name
                        scoreLogical,       //its logical volume
               sphereLogical,               //its mother logical
                       kZAxis,              //axis
                       nRings,              //number of copies
                       param);              //parameterisation                            
////////////////////////////////////////////////////////////////////////
// Visualisation attributes
  
  // Invisible world volume.
  fpWorldLogical->SetVisAttributes(G4VisAttributes::Invisible);

  // Beam Window - brown
  G4VisAttributes* beamWindowAttributes = new G4VisAttributes(G4Colour(0.5,0.0,0.0,1.0));
  beamWindowAttributes->SetForceSolid(true);
  beamWindowLogical->SetVisAttributes(beamWindowAttributes);
  //
  // Beam Pipe Vacuum - yellow
  G4VisAttributes* beamPipeAttributes = new G4VisAttributes(G4Colour::Yellow());
  //beamPipeAttributes->SetForceSolid(true);
  beamPipeLogical->SetVisAttributes(beamPipeAttributes);
 
  // Target -  light blue
  G4VisAttributes* targetAttributes = new G4VisAttributes(G4Colour(0.0,0.5,0.5,1.0));
  targetAttributes->SetForceSolid(true);
  targetLogical->SetVisAttributes(targetAttributes);
  //
  // Chamber window - magenta
  G4VisAttributes* chamberAttributes = new G4VisAttributes(G4Colour::Magenta());
  chamberAttributes->SetForceSolid(true);
  chamberLogical->SetVisAttributes(chamberAttributes);
  //
  // Silicon monitor - grey
  G4VisAttributes* monitorAttributes = new G4VisAttributes(G4Colour::Grey());
  monitorAttributes->SetForceSolid(true);
  monitorLogical->SetVisAttributes(monitorAttributes);   
  // Sphere - white
  G4VisAttributes* sphereAttributes =
        new G4VisAttributes(G4Colour(1.0,1.0,1.0,0.5));
  sphereAttributes->SetVisibility(false);
  sphereLogical->SetVisAttributes(sphereAttributes);
  // Score - green
  G4VisAttributes* scoreAttributes =
        new G4VisAttributes(G4Colour(0.0,1.0,0.0,0.5));
  scoreLogical->SetVisAttributes(scoreAttributes);
}
